@@ -73,12 +73,15 @@ Bitu vga_read_p3d4(Bitu port,Bitu iolen) {
     (void)port;//UNUSED
     (void)iolen;//UNUSED
 
-    /* NOTES: Paradise/Westdern Digital SVGA decodes only bits [5:0] inclusive and repeat every 0x40 */
+    /* NOTES: Paradise/Western Digital SVGA decodes only bits [5:0] inclusive and repeat every 0x40 */
 
 	return crtc(index);
 }
 
 void vga_write_p3d5(Bitu port,Bitu val,Bitu iolen) {
+	if (vga.dosboxig.vga_reg_lockout)
+		return;
+
     (void)port;//UNUSED
 //	if((crtc(index)!=0xe)&&(crtc(index)!=0xf)) 
 //		LOG_MSG("CRTC w #%2x val %2x",crtc(index),val);
@@ -242,18 +245,21 @@ void vga_write_p3d5(Bitu port,Bitu val,Bitu iolen) {
 		*/
 		break;
 	case 0x0C:	/* Start Address High Register */
+		vga.draw.must_draw_again = true; // takes effect NEXT frame, not current frame!
 		crtc(start_address_high)=(uint8_t)val;
 		vga.config.display_start=(vga.config.display_start & 0xFF00FF)| (val << 8);
 		/* 0-7  Upper 8 bits of the start address of the display buffer */
 		page_flip_debug_notify();
 		break;
 	case 0x0D:	/* Start Address Low Register */
+		vga.draw.must_draw_again = true; // takes effect NEXT frame, not current frame!
 		crtc(start_address_low)=(uint8_t)val;
 		vga.config.display_start=(vga.config.display_start & 0xFFFF00)| val;
 		/*	0-7	Lower 8 bits of the start address of the display buffer */
 		page_flip_debug_notify();
 		break;
 	case 0x0E:	/*Cursor Location High Register */
+		if (vga_render_on_demand) VGA_RenderOnDemandUpTo();
 		crtc(cursor_location_high)=(uint8_t)val;
 		vga.config.cursor_start&=0xff00ff;
 		vga.config.cursor_start|=val << 8;
@@ -262,7 +268,7 @@ void vga_write_p3d5(Bitu port,Bitu val,Bitu iolen) {
 		 * color index [http://hackipedia.org/browse.cgi/Computer/Platform/PC%2c%20IBM%20compatible/Video/VGA/SVGA/S3%20Graphics%2c%20Ltd/S3%2086C928%20GUI%20Accelerator%20%281992%2d09%29%2epdf] */
 		break;
 	case 0x0F:	/* Cursor Location Low Register */
-		//TODO update cursor on screen
+		if (vga_render_on_demand) VGA_RenderOnDemandUpTo();
 		crtc(cursor_location_low)=(uint8_t)val;
 		vga.config.cursor_start&=0xffff00;
 		vga.config.cursor_start|=val;
@@ -331,6 +337,7 @@ void vga_write_p3d5(Bitu port,Bitu val,Bitu iolen) {
 		*/
 		break;
 	case 0x14:	/* Underline Location Register */
+		if (vga_render_on_demand) VGA_RenderOnDemandUpTo();
 		crtc(underline_location)=(uint8_t)val;
 		VGA_CheckAddrShift();
 		VGA_CheckScanLength();
@@ -366,14 +373,9 @@ void vga_write_p3d5(Bitu port,Bitu val,Bitu iolen) {
 		if (vga_render_on_demand) VGA_RenderOnDemandUpTo();
 		crtc(mode_control)=(uint8_t)val;
 		vga.tandy.line_mask = (~val) & 3u;
-
-		if ( vga.tandy.line_mask ) {
-			vga.tandy.line_shift = 13u;
-			vga.tandy.addr_mask = (1u << 13u) - 1u;
-		} else {
-			vga.tandy.addr_mask = ~0u;
-			vga.tandy.line_shift = 0;
-		}
+		vga.tandy.line_shift = vga.tandy.line_mask ? 13u : 0u;
+		vga.tandy.addr_mask = ~0u;
+		vga.tandy.addr_mask &= ~(vga.tandy.line_mask << 13u);
 
 		VGA_CheckAddrShift();
 		VGA_CheckScanLength();
