@@ -18,7 +18,9 @@
  * 02110-1301, USA
  */
 
-#if !C_FLUIDSYNTH && defined(WIN32)
+#define FLUIDINC
+#include "config.h"
+#if !C_FLUIDSYNTH && defined(WIN32) && !defined(C_HX_DOS)
 #include "fluid_ramsfont.h"
 #include "fluid_sys.h"
 #include "fluid_synth.h"
