@@ -1437,7 +1437,8 @@ char *QZ_GetIMValues(_THIS, SDL_imvalue value, int *alt)
             *alt = GetEnableIME();
             return NULL;
         case SDL_IM_FONT_SIZE:
-            *alt = [field_edit getFontHeight];
+            // 2024-03-28 Suddenly XCode considers id different from int and throws an error here without typecast
+            *alt = (int)[field_edit getFontHeight];
             return NULL;
         default:
             SDL_SetError("QZ_GetIMValues: nuknown enum type %d", value);
@@ -1456,7 +1457,8 @@ int QZ_FlushIMString(_THIS, void *buffer)
             *(char *)buffer = 0;
         }
     }
-    return [field_edit getTextLength];
+    // 2024-03-28 Suddenly XCode considers id different from int and throws an error here without typecast
+    return (int)[field_edit getTextLength];
 }
 
 #else /* !ENABLE_IM_EVENT */
